from .convert import to_zarr
import fire
import json
import nd2
import os

def main(nd2_path:str, output:str=None, channel_axis:int=1, steps:int=5, dry_run=False):
    data = (d := nd2.ND2File(nd2_path)).to_dask().rechunk()
    print(d.sizes)

    try:
        channel_axis = list(d.sizes.keys()).index('C')
    except ValueError:
        channel_axis = None

    if output is None:
        output = nd2_path.replace('.nd2', '.zarr')

        
    out = to_zarr(
        data, 
        output, 
        steps=steps, 
        dry_run=dry_run, 
        channel_axis=channel_axis, 
        sizes=d.sizes,
        lut=get_lut(d))
    
    assert os.path.exists(out), "Saving zarr failed..."
    
    try:
        meta_path = save_metadata(d, output)
        print(f'Saved metadata to {meta_path}')
    except Exception as e:
        print(f'Saving metadata Failed due to {e}')
        
    exit(0)

def save_metadata(data:nd2.ND2File, output:str):
    try:
        meta = data.custom_data
        meta['NDControlV1_0'] = {}
        with open(meta_path:=os.path.join(output, '.metadata'), 'w') as f:
            json.dump(meta, f, indent=4)
    except TypeError as e:
        raise e
    return meta_path

def get_lut(data:nd2.ND2File):
    try:
        meta = data.custom_data
        return \
        [
            [
                meta['LUTDataV1_0']['LutParam']['CompLutParam'][f'{ch:02d}'][lim][0] \
                for lim in ['MinSrc','MaxSrc']
            ] for ch in range(data.sizes['C'])
        ]
    except Exception as e:
        print(f'Unable to get LUT due to {e}')
        return None

if __name__=="__main__":
    fire.Fire(main)



import zarr
import dask.array as da
import os

def to_zarr(
    dask_input:da.Array, 
    path:str=None, 
    steps=3, 
    channel_axis=1, 
    dry_run=False, 
    **kwargs
):
    store = zarr.DirectoryStore(baseurl := path)
    grp = zarr.group(store)
    
    print(baseurl)
    datasets = []
    for i in range(steps):
        if i==0:
            data = dask_input
        else:
            data = dask_input[...,::2,::2]

        try:
            data = data.rechunk()
            print(data.chunksize, data.shape, data.npartitions)
            if dry_run:
                print(f'dry-run `to save to` {os.path.join(baseurl, str(i))}')
                continue
            
            data.to_zarr(ppp:=os.path.join(baseurl, str(i)))
            print(f'saved {ppp}')
            datasets.append({"path": str(i)})
            grp.attrs['multiscales'] = {
                "multiscales": [
                    {
                        "datasets": datasets,
                        "title": os.path.basename(baseurl),
                        "type": "nd2",
                        "channel_axis": channel_axis,
                        "version": "0.1",
                        **kwargs
                    },

                ]
            }
            dask_input = da.from_zarr(ppp)
        except Exception as e:
            print(e.args)
            raise e
        
    return baseurl
    




from zarr_tools import convert, __main__
import numpy as np
import dask.array as da
import os
import shutil

def test_convert():

    data = np.zeros((15,3,2**12, 2**12), dtype="uint16")
    path = "test.zarr"
    convert.to_zarr(
        da.from_array(data),
        path = path,
        steps=3
    )
    for i in range(3):
        assert da.from_zarr(os.path.join(path, str(i))).shape[-1] == 2**(12-i)
    
    shutil.rmtree(path)